@@ -80,45 +80,30 @@ static const uint32_t wep_crc32_table[256] = {
     0x2d02ef8dL
 };
 
-// Munge text down to printable characters only
-void MungeToPrintable(char *in_data, int max) {
-    unsigned char *temp = new unsigned char[max];
-    strncpy((char *) temp, in_data, max);
-    // Make sure we terminate this just in case
+// Munge text down to printable characters only.  Simpler, cleaner munger than
+// before (and more blatant when munging)
+string MungeToPrintable(char *in_data, int len) {
+	string ret;
+	int i;
 
-    int i, j;
+	for (i = 0; i < len; i++) {
+		if ((unsigned char) in_data[i] >= 32 && (unsigned char) in_data[i] <= 176) {
+			ret += in_data[i];
+		} else {
+			ret += '\\';
+			ret += ((in_data[i] >> 6) & 0x03) + '0';
+			ret += ((in_data[i] >> 3) & 0x07) + '0';
+			ret += ((in_data[i] >> 0) & 0x07) + '0';
+		}
+	}
 
-    for (i = 0, j = 0; i < max && j < max; i++) {
-        if (temp[i] == 0 || temp[i] == '\n') {
-            in_data[j++] = '\0';
-            break;
-        }
-
-        if (temp[i] < 32) {
-            if (temp[i] < 32) {
-                if (j+2 < max) {
-                    // Convert control chars to ^X and so on
-                    in_data[j++] = '^';
-                    in_data[j++] = temp[i] + 64;
-                } else {
-                    break;
-                }
-            }
-        } else if (temp[i] > 126) {
-            // Do nothing
-        } else {
-            in_data[j++] = temp[i];
-        }
-    }
-    in_data[j] = '\0';
-
-    delete[] temp;
+	return ret;
 }
 
-
 // Returns a pointer in the data block to the size byte of the desired
 // tag.
-int GetTagOffsets(int init_offset, kis_packet *packet, map<int, int> *tag_cache_map) {
+int GetTagOffsets(int init_offset, kis_packet *packet, 
+				  map<int, vector<int> > *tag_cache_map) {
     int cur_tag = 0;
     // Initial offset is 36, that's the first tag
     int cur_offset = init_offset;
@@ -148,16 +133,9 @@ int GetTagOffsets(int init_offset, kis_packet *packet, map<int, int> *tag_cache_
                 return -1;
             }
 
-            (*tag_cache_map)[cur_tag] = cur_offset + 1;
-
-            /*
-               if (cur_tag == tagnum) {
-               cur_offset++;
-               break;
-               } else if (cur_tag > tagnum) {
-               return -1;
-               }
-               */
+            // (*tag_cache_map)[cur_tag] = cur_offset + 1;
+			
+            (*tag_cache_map)[cur_tag].push_back(cur_offset + 1);
 
             // Jump the length+length byte, this should put us at the next tag
             // number.
@@ -168,6 +146,52 @@ int GetTagOffsets(int init_offset, kis_packet *packet, map<int, int> *tag_cache_
     return 0;
 }
 
+int WPACipherConv(uint8_t cipher_index) {
+	int ret = crypt_wpa;
+
+	switch (cipher_index) {
+		case 1:
+			ret |= crypt_wep40;
+			break;
+		case 2:
+			ret |= crypt_tkip;
+			break;
+		case 3:
+			ret |= crypt_aes_ocb;
+			break;
+		case 4:
+			ret |= crypt_aes_ccm;
+			break;
+		case 5:
+			ret |= crypt_wep104;
+			break;
+		default:
+			ret = 0;
+			break;
+	}
+
+	return ret;
+}
+
+int WPAKeyMgtConv(uint8_t mgt_index) {
+	int ret = crypt_wpa;
+
+	switch (mgt_index) {
+		case 1:
+			ret |= crypt_wpa;
+			break;
+		case 2:
+			ret |= crypt_psk;
+			break;
+		default:
+			ret = 0;
+			break;
+	}
+
+	return ret;
+}
+
+
 // Get the info from a packet
 static int dissect_packet_num = 0;
 void GetPacketInfo(kis_packet *packet, packet_info *ret_packinfo,
@@ -295,7 +319,8 @@ void GetPacketInfo(kis_packet *packet, packet_info *ret_packinfo,
         } else {
             ret_packinfo->header_offset = 36;
             fixparm = (fixed_parameters *) &packet->data[24];
-            ret_packinfo->wep = fixparm->wep;
+			if (fixparm->wep)
+				ret_packinfo->crypt_set |= crypt_wep;
 
             // Pull the fixparm ibss info
             if (fixparm->ess == 0 && fixparm->ibss == 1) {
@@ -313,8 +338,8 @@ void GetPacketInfo(kis_packet *packet, packet_info *ret_packinfo,
 #endif
         }
 
-        map<int, int> tag_cache_map;
-        map<int, int>::iterator tcitr;
+        map<int, vector<int> > tag_cache_map;
+        map<int, vector<int> >::iterator tcitr;
 
         // Extract various tags from the packet
         int found_ssid_tag = 0;
@@ -331,7 +356,7 @@ void GetPacketInfo(kis_packet *packet, packet_info *ret_packinfo,
             }
       
             if ((tcitr = tag_cache_map.find(0)) != tag_cache_map.end()) {
-                tag_offset = tcitr->second;
+                tag_offset = tcitr->second[0];
 
                 found_ssid_tag = 1;
                 temp = (packet->data[tag_offset] & 0xFF);
@@ -340,11 +365,24 @@ void GetPacketInfo(kis_packet *packet, packet_info *ret_packinfo,
                 if (temp == 0) {
                     // do nothing for 0-length ssid's
                 } else if (temp <= 32) {
-                    memcpy(ret_packinfo->ssid, &packet->data[tag_offset+1], temp);
-                    ret_packinfo->ssid[temp] = '\0';
-                    // Munge it down to printable characters... SSID's can be anything
-                    // but if we can't print them it's not going to be very useful
-                    MungeToPrintable(ret_packinfo->ssid, temp);
+					// Look for a non-0 length but all zero content SSID used
+					// by some systems for cloaking.  Don't waste time copying
+					// what doesn't need to be copied, and munge the rest into
+					// something safe
+					int zeroed = 1;
+					for (unsigned int sp = 0; sp < temp; sp++) {
+						if (packet->data[tag_offset + 1 + sp] != 0) {
+							zeroed = 0;
+							break;
+						}
+					}
+
+					if (zeroed == 0) {
+						snprintf(ret_packinfo->ssid, SSID_SIZE, "%s",
+								 MungeToPrintable((char *)
+												  &(packet->data[tag_offset + 1]), 
+												  temp).c_str());
+					}
                 } else {
                     // Otherwise we're corrupt, set it and stop processing
                     ret_packinfo->corrupt = 1;
@@ -356,7 +394,7 @@ void GetPacketInfo(kis_packet *packet, packet_info *ret_packinfo,
 
             // Extract the supported rates
             if ((tcitr = tag_cache_map.find(1)) != tag_cache_map.end()) {
-                tag_offset = tcitr->second;
+                tag_offset = tcitr->second[0];
 
                 found_rate_tag = 1;
                 for (int x = 0; x < packet->data[tag_offset]; x++) {
@@ -371,7 +409,7 @@ void GetPacketInfo(kis_packet *packet, packet_info *ret_packinfo,
             // this tag so we use the hardware channel, assigned at the beginning of 
             // GetPacketInfo
             if ((tcitr = tag_cache_map.find(3)) != tag_cache_map.end()) {
-                tag_offset = tcitr->second;
+                tag_offset = tcitr->second[0];
                 found_channel_tag = 1;
                 // Extract the channel from the next byte (GetTagOffset returns
                 // us on the size byte)
@@ -421,7 +459,7 @@ void GetPacketInfo(kis_packet *packet, packet_info *ret_packinfo,
                 ret_packinfo->corrupt = 1;
 
             // Catch wellenreiter probes
-            if (!strncmp(ret_packinfo->ssid, "this_is_used_for_wellenreiter", 32)) {
+            if (!strncmp(ret_packinfo->ssid, "this_is_used_for_wellenreiter", 29)) {
                 ret_packinfo->proto.type = proto_wellenreiter;
             }
 
@@ -448,24 +486,161 @@ void GetPacketInfo(kis_packet *packet, packet_info *ret_packinfo,
 
             // Extract the CISC.O beacon info
             if ((tcitr = tag_cache_map.find(133)) != tag_cache_map.end()) {
-                tag_offset = tcitr->second;
+                tag_offset = tcitr->second[0];
+                temp = (packet->data[tag_offset] & 0xFF);
 
-                if ((unsigned) tag_offset + 11 < packet->len) {
-                    snprintf(ret_packinfo->beacon_info, BEACON_INFO_LEN, "%s", &packet->data[tag_offset+11]);
-                    MungeToPrintable(ret_packinfo->beacon_info, BEACON_INFO_LEN);
-                } else {
-                    // Otherwise we're corrupt, bail
-                    ret_packinfo->corrupt = 1;
-                    return;
+				// Copy and munge the beacon info if it falls w/in our
+				// boundaries
+				if ((unsigned int) (tag_offset + 11) < packet->len && temp >= 11) {
+					snprintf(ret_packinfo->beacon_info, SSID_SIZE,
+						MungeToPrintable((char *) &(packet->data[tag_offset+11]), temp - 11).c_str());
                 }
+
+				// Non-fatal fail since beacon info might not have that
+				// 11 byte leader on it, I don't know
             }
 
-			// Extract WPA info
-			if ((tcitr = tag_cache_map.find(221)) != tag_cache_map.end()) {
-				// Assume we have WPA at all right now
-				ret_packinfo->wpa = 1;
+			// WPA frame matching if we have the privacy bit set
+			if ((ret_packinfo->crypt_set & crypt_wep)) {
+				// Liberally borrowed from Ethereal
+				if ((tcitr = tag_cache_map.find(221)) != tag_cache_map.end()) {
+					for (unsigned int tagct = 0; tagct < tcitr->second.size(); 
+						 tagct++) {
+						tag_offset = tcitr->second[tagct];
+						unsigned int tag_orig = tag_offset + 1;
+						unsigned int taglen = (packet->data[tag_offset] & 0xFF);
+						unsigned int offt = 0;
+
+						if (tag_orig + taglen > packet->len) {
+							ret_packinfo->corrupt = 1;
+							return;
+						}
+
+						// Match 221 tag header for WPA
+						if (taglen < 6 || memcmp(&(packet->data[tag_orig + offt]), 
+												 WPA_OUI, sizeof(WPA_OUI)))
+							continue;
+
+						offt += 6;
+
+						// Match WPA multicast suite
+						if (offt + 4 > taglen || 
+							memcmp(&(packet->data[tag_orig + offt]), WPA_OUI,
+								   sizeof(WPA_OUI)))
+							continue;
+
+						ret_packinfo->crypt_set |= 
+							WPACipherConv(packet->data[tag_orig + offt + 3]);
+
+						// We don't care about parsing the number of ciphers,
+						// we'll just iterate, so skip the cipher number
+						offt += 6;
+
+						// Match WPA unicast components
+						while (offt + 4 <= taglen) {
+							if (memcmp(&(packet->data[tag_orig + offt]), 
+									  WPA_OUI, sizeof(WPA_OUI)) == 0) {
+								ret_packinfo->crypt_set |= 
+									WPACipherConv(packet->data[tag_orig + offt + 3]);
+								offt += 4;
+							} else {
+								break;
+							}
+						}
+
+						// Match auth key components
+						offt += 2;
+						while (offt + 4 <= taglen) {
+							if (memcmp(&(packet->data[tag_orig + offt]), 
+									  WPA_OUI, sizeof(WPA_OUI)) == 0) {
+								ret_packinfo->crypt_set |= 
+									WPACipherConv(packet->data[tag_orig + offt + 3]);
+								offt += 4;
+							} else {
+								break;
+							}
+						}
+					}
+				} /* 221 */
+
+				// Match tag 48 RSN WPA2
+				if ((tcitr = tag_cache_map.find(48)) != tag_cache_map.end()) {
+					for (unsigned int tagct = 0; tagct < tcitr->second.size(); 
+						 tagct++) {
+						tag_offset = tcitr->second[tagct];
+						unsigned int tag_orig = tag_offset + 1;
+						unsigned int taglen = (packet->data[tag_offset] & 0xFF);
+						unsigned int offt = 0;
+
+						if (tag_orig + taglen > packet->len || taglen < 6) {
+							ret_packinfo->corrupt = 1;
+							return;
+						}
+
+						// Skip version
+						offt += 2;
+
+						// Match multicast
+						if (offt + 3 > taglen ||
+							memcmp(&(packet->data[tag_orig + offt]), RSN_OUI,
+								   sizeof(RSN_OUI))) {
+							ret_packinfo->corrupt = 1;
+							return;
+						}
+						ret_packinfo->crypt_set |= 
+							WPACipherConv(packet->data[tag_orig + offt + 3]);
+						offt += 4;
+
+						// We don't care about unicast number
+						offt += 2;
+
+						while (offt + 4 <= taglen) {
+							if (memcmp(&(packet->data[tag_orig + offt]), 
+									  RSN_OUI, sizeof(RSN_OUI)) == 0) {
+								ret_packinfo->crypt_set |= 
+									WPACipherConv(packet->data[tag_orig + offt + 3]);
+								offt += 4;
+							} else {
+								break;
+							}
+						}
+
+						// We don't care about authkey number
+						offt += 2;
+
+						while (offt + 4 <= taglen) {
+							if (memcmp(&(packet->data[tag_orig + offt]), 
+									  RSN_OUI, sizeof(RSN_OUI)) == 0) {
+								ret_packinfo->crypt_set |= 
+									WPAKeyMgtConv(packet->data[tag_orig + offt + 3]);
+								offt += 4;
+							} else {
+								break;
+							}
+						}
+					}
+				} /* 48 */
 			}
 
+# if 0
+			// Extract WPA info -- we have to look at all the tags
+			if ((tcitr = tag_cache_map.find(48)) != tag_cache_map.end() &&
+				(ret_packinfo->crypt_set & crypt_wep)) {
+				for (unsigned int tagct = 0; tagct < tcitr->second.size(); tagct++) {
+					tag_offset = tcitr->second[tagct];
+					temp = (packet->data[tag_offset] & 0xFF);
+
+					if (temp > 6 && 
+						memcmp(&(packet->data[tag_offset+1]), 
+							   RSN_AES_TAGPARM_SIGNATURE,
+							   sizeof(RSN_AES_TAGPARM_SIGNATURE)) == 0) {
+						(int) ret_packinfo->crypt_set |= crypt_wpa2aes;
+						break;
+					}
+				}
+			}
+#endif
+
             ret_packinfo->dest_mac = addr0;
             ret_packinfo->source_mac = addr1;
             ret_packinfo->bssid_mac = addr2;
@@ -646,10 +821,17 @@ void GetPacketInfo(kis_packet *packet, packet_info *ret_packinfo,
             break;
         default:
             ret_packinfo->corrupt = 1;
+			ret_packinfo->header_offset = 0;
             return;
             break;
         }
 
+		// Check the header bounds
+		if (packet->len < ret_packinfo->header_offset) {
+			ret_packinfo->corrupt = 1;
+			return;
+		}
+
 		// If we're special data frame types bail now
 		if (ret_packinfo->subtype != packet_sub_data) {
 			ret_packinfo->datasize = 0;
@@ -658,77 +840,30 @@ void GetPacketInfo(kis_packet *packet, packet_info *ret_packinfo,
 
         // Detect encrypted frames
         if (fc->wep &&
-            (*((unsigned short *) &packet->data[ret_packinfo->header_offset]) != 0xAAAA ||
-             packet->data[ret_packinfo->header_offset + 1] & 0x40)) {
-
+			(*((unsigned short *) &packet->data[ret_packinfo->header_offset]) != 
+			 0xAAAA || packet->data[ret_packinfo->header_offset + 1] & 0x40)) {
             ret_packinfo->encrypted = 1;
-        } else if (packet->parm.fuzzy_crypt && 
-                   (unsigned int) ret_packinfo->header_offset+9 < packet->len) {
-            // Do a fuzzy data compare... if it's not:
-            // 0xAA - IP LLC
-            // 0x42 - I forgot.
-            // 0xF0 - Netbios
-            // 0xE0 - IPX
-            if (packet->data[ret_packinfo->header_offset] != 0xAA && packet->data[ret_packinfo->header_offset] != 0x42 &&
-                packet->data[ret_packinfo->header_offset] != 0xF0 && packet->data[ret_packinfo->header_offset] != 0xE0) {
-                ret_packinfo->encrypted = 1;
-                ret_packinfo->fuzzy = 1;
-            }
-        }
+		}
 
-        if (ret_packinfo->encrypted) {
-            // Match the range of cryptographically weak packets and let us
-            // know.
-
-            // New detection method from Airsnort 2.0, should be much better.
-            unsigned int sum;
-            if (packet->data[ret_packinfo->header_offset+1] == 255 && packet->data[ret_packinfo->header_offset] > 2 &&
-                packet->data[ret_packinfo->header_offset] < 16) {
-                ret_packinfo->interesting = 1;
-            } else {
-                sum = packet->data[ret_packinfo->header_offset] + packet->data[ret_packinfo->header_offset+1];
-                if (sum == 1 && (packet->data[ret_packinfo->header_offset + 2] <= 0x0A ||
-                                 packet->data[ret_packinfo->header_offset + 2] == 0xFF)) {
-                    ret_packinfo->interesting = 1;
-                } else if (sum <= 0x0C && (packet->data[ret_packinfo->header_offset + 2] >= 0xF2 &&
-                                           packet->data[ret_packinfo->header_offset + 2] <= 0xFE &&
-                                           packet->data[ret_packinfo->header_offset + 2] != 0xFD))
-                    ret_packinfo->interesting = 1;
-            }
-
-            // Knock 8 bytes off the data size of encrypted packets for the
-            // wep IV and check
-            datasize = ret_packinfo->datasize - 8;
-            if (datasize > 0)
-                ret_packinfo->datasize = datasize;
-            else
-                ret_packinfo->datasize = 0;
-
-            if (ret_packinfo->encrypted) {
-                // De-wep if we have any keys
-                if (bssid_wep_map->size() != 0)
-                    DecryptPacket(packet, ret_packinfo, bssid_wep_map, identity);
-
-                // Record the IV in the info
-                memcpy(&ret_packinfo->ivset, 
-                       &packet->data[ret_packinfo->header_offset], 4);
-            }
-
-        }
+		// If we're known encrypted or if we want to be fuzzycrypt detected, 
+		// process the crypto info.  Otherwise we don't.  Cryptoinfo might
+		// later be processed again if we're doing network-classification
+		// wep detection
+		if (ret_packinfo->encrypted || packet->parm.fuzzy_crypt)
+			ProcessPacketCrypto(packet, ret_packinfo, bssid_wep_map, identity);
 
+		// Extract the tcp layer info
         if (ret_packinfo->encrypted == 0 || ret_packinfo->decoded == 1)
             GetProtoInfo(packet, ret_packinfo);
 
     } else {
-        // If we didn't figure out what it was yet, test it for cisco noise.  If the first
-        // four bytes are 0xFF, bail
+        // If we didn't figure out what it was yet, test it for cisco noise.  
+		// If the first four bytes are 0xFF, bail
         uint32_t fox = ~0;
         if (memcmp(packet->data, &fox, 4) == 0) {
             ret_packinfo->type = packet_noise;
             return;
         }
-
-
     }
 
     // Do a little sanity checking on the BSSID
@@ -740,6 +875,73 @@ void GetPacketInfo(kis_packet *packet, packet_info *ret_packinfo,
 
 }
 
+// Handle detecting crypted packets and decoding them
+void ProcessPacketCrypto(kis_packet *packet, packet_info *ret_packinfo,
+						 macmap<wep_key_info *> *bssid_wep_map,
+						 unsigned char *identity) {
+	int datasize = ret_packinfo->datasize;
+
+	if (ret_packinfo->encrypted == 0 && 
+		(unsigned int) ret_packinfo->header_offset+9 < packet->len) {
+		// Do a fuzzy data compare... if it's not:
+		// 0xAA - IP LLC
+		// 0x42 - I forgot.
+		// 0xF0 - Netbios
+		// 0xE0 - IPX
+		if (packet->data[ret_packinfo->header_offset] != 0xAA && 
+			packet->data[ret_packinfo->header_offset] != 0x42 &&
+			packet->data[ret_packinfo->header_offset] != 0xF0 && 
+			packet->data[ret_packinfo->header_offset] != 0xE0) {
+			ret_packinfo->encrypted = 1;
+			ret_packinfo->fuzzy = 1;
+		}
+	}
+
+	if (ret_packinfo->encrypted) {
+		// Match the range of cryptographically weak packets and let us
+		// know.
+
+		// New detection method from Airsnort 2.0, should be much better.
+		unsigned int sum;
+		if (packet->data[ret_packinfo->header_offset+1] == 255 && 
+			packet->data[ret_packinfo->header_offset] > 2 &&
+			packet->data[ret_packinfo->header_offset] < 16) {
+			ret_packinfo->interesting = 1;
+		} else {
+			sum = packet->data[ret_packinfo->header_offset] + 
+				packet->data[ret_packinfo->header_offset+1];
+			if (sum == 1 && 
+				(packet->data[ret_packinfo->header_offset + 2] <= 0x0A ||
+				 packet->data[ret_packinfo->header_offset + 2] == 0xFF)) {
+				ret_packinfo->interesting = 1;
+			} else if (sum <= 0x0C && 
+					   (packet->data[ret_packinfo->header_offset + 2] >= 0xF2 &&
+						packet->data[ret_packinfo->header_offset + 2] <= 0xFE &&
+						packet->data[ret_packinfo->header_offset + 2] != 0xFD))
+				ret_packinfo->interesting = 1;
+		}
+
+		// Knock 8 bytes off the data size of encrypted packets for the
+		// wep IV and check
+		datasize = ret_packinfo->datasize - 8;
+		if (datasize > 0)
+			ret_packinfo->datasize = datasize;
+		else
+			ret_packinfo->datasize = 0;
+
+		if (ret_packinfo->encrypted) {
+			// De-wep if we have any keys
+			if (bssid_wep_map->size() != 0)
+				DecryptPacket(packet, ret_packinfo, bssid_wep_map, identity);
+
+			// Record the IV in the info
+			memcpy(&ret_packinfo->ivset, 
+				   &packet->data[ret_packinfo->header_offset], 4);
+		}
+
+	}
+}
+
 void GetProtoInfo(kis_packet *packet, packet_info *in_info) {
     /* This buffering is an unpleasant slowdown, so lets trust our code to be correct
      * now. */
@@ -772,33 +974,52 @@ void GetProtoInfo(kis_packet *packet, packet_info *in_info) {
                    sizeof(PROBE_LLC_SIGNATURE)) == 0) {
 
             // If we have a LLC packet that looks like a netstumbler...
-            if (memcmp(&data[in_info->header_offset + NETSTUMBLER_OFFSET], NETSTUMBLER_322_SIGNATURE,
+            if (memcmp(&data[in_info->header_offset + NETSTUMBLER_OFFSET], 
+					   NETSTUMBLER_322_SIGNATURE,
                        sizeof(NETSTUMBLER_322_SIGNATURE)) == 0) {
                 // Netstumbler 322 says Flurble gronk bloopit, bnip Frundletrune
                 ret_protoinfo->type = proto_netstumbler;
                 ret_protoinfo->prototype_extra = 22;
                 return;
-            } else if (memcmp(&data[in_info->header_offset + NETSTUMBLER_OFFSET], NETSTUMBLER_323_SIGNATURE,
+            } else if (memcmp(&data[in_info->header_offset + NETSTUMBLER_OFFSET], 
+							  NETSTUMBLER_323_SIGNATURE,
                               sizeof(NETSTUMBLER_323_SIGNATURE)) == 0) {
                 // Netstumbler 323 says All your 802.11b are belong to us
                 ret_protoinfo->type = proto_netstumbler;
                 ret_protoinfo->prototype_extra = 23;
                 return;
-            } else if (memcmp(&data[in_info->header_offset + NETSTUMBLER_OFFSET], NETSTUMBLER_330_SIGNATURE,
+            } else if (memcmp(&data[in_info->header_offset + NETSTUMBLER_OFFSET], 
+							  NETSTUMBLER_330_SIGNATURE,
                               sizeof(NETSTUMBLER_330_SIGNATURE)) == 0) {
                 // Netstumbler 330 says           Intentionally left blank
                 ret_protoinfo->type = proto_netstumbler;
                 ret_protoinfo->prototype_extra = 30;
                 return;
-            } else if (memcmp(&data[in_info->header_offset + LUCENT_OFFSET], LUCENT_TEST_SIGNATURE,
+            } else if (memcmp(&data[in_info->header_offset + LUCENT_OFFSET], 
+							  LUCENT_TEST_SIGNATURE,
                               sizeof(LUCENT_TEST_SIGNATURE)) == 0) {
                 ret_protoinfo->type = proto_lucenttest;
             }
-        } else if (memcmp(&data[in_info->header_offset + LLC_UI_OFFSET], CISCO_SIGNATURE,
-               sizeof(CISCO_SIGNATURE)) == 0) {
+        } else if (memcmp(&data[in_info->header_offset + LLC_UI_OFFSET], 
+						  CISCO_SIGNATURE, sizeof(CISCO_SIGNATURE)) == 0) {
             // CDP
 
-            unsigned int offset = in_info->header_offset + LLC_UI_OFFSET + 12;
+			// What was this?  I don't know why I used +12.  Test the version,
+			// now.  If it's v2, do what should be right, otherwise leave the
+			// brokenness.
+			if (packet->len < (unsigned int) in_info->header_offset + 
+				LLC_UI_OFFSET + sizeof(CISCO_SIGNATURE)) {
+				return;
+			}
+
+			unsigned int offset = 0;
+
+			if (packet->data[in_info->header_offset + LLC_UI_OFFSET + 
+				sizeof(CISCO_SIGNATURE)] == 2)
+				offset = in_info->header_offset + LLC_UI_OFFSET + 
+					sizeof(CISCO_SIGNATURE) + 4;
+			else
+				offset = in_info->header_offset + LLC_UI_OFFSET + 12;
 
             while (offset < packet->len) {
                 // Make sure that whatever we do, we don't wander off the
@@ -807,12 +1028,25 @@ void GetProtoInfo(kis_packet *packet, packet_info *in_info) {
 
                 cdp_element *elem = (cdp_element *) &data[offset];
 
+				elem->length = kis_ntoh16(elem->length);
+				elem->type = kis_ntoh16(elem->type);
+
                 if (elem->length == 0)
                     break;
 
+				if (offset + elem->length >= packet->len) {
+					break;
+				}
+
                 if (elem->type == 0x01) {
                     // Device id
-                    snprintf(ret_protoinfo->cdp.dev_id, elem->length-3, "%s", (char *) &elem->data);
+					if (elem->length < 3) {
+						fprintf(stderr, "*** Warning - Corrupt CDP frame (possibly "
+								"an exploit attempt. (0x01 frame < 3)\n");
+						return;
+					}
+                    snprintf(ret_protoinfo->cdp.dev_id, elem->length-3, "%s", 
+							 (char *) &elem->data);
                 } else if (elem->type == 0x02) {
                     // IP range
 
@@ -829,16 +1063,39 @@ void GetProtoInfo(kis_packet *packet, packet_info *in_info) {
                     // }
                 } else if (elem->type == 0x03) {
                     // port id
-                    snprintf(ret_protoinfo->cdp.interface, elem->length-3, "%s", (char *) &elem->data);
+					if (elem->length < 3) {
+						fprintf(stderr, "*** Warning - Corrupt CDP frame (possibly "
+								"an exploit attempt. (0x03 frame < 3)\n");
+						return;
+					}
+                    snprintf(ret_protoinfo->cdp.interface, elem->length-3, "%s", 
+							 (char *) &elem->data);
                 } else if (elem->type == 0x04) {
                     // capabilities
+					if (elem->length < 4) {
+						fprintf(stderr, "*** Warning - Corrupt CDP frame (possibly "
+								"an exploit attempt. (0x04 frame < 4)\n");
+						return;
+					}
                     memcpy(&ret_protoinfo->cdp.cap, &elem->data, elem->length-4);
                 } else if (elem->type == 0x05) {
                     // software version
-                    snprintf(ret_protoinfo->cdp.software, elem->length-3, "%s", (char *) &elem->data);
+					if (elem->length <= 3) {
+						fprintf(stderr, "*** Warning - Corrupt CDP frame (possibly "
+								"an exploit attempt. (0x05 frame< 3)\n");
+						return;
+					}
+                    snprintf(ret_protoinfo->cdp.software, elem->length-3, "%s", 
+							 (char *) &elem->data);
                 } else if (elem->type == 0x06) {
                     // Platform
-                    snprintf(ret_protoinfo->cdp.platform, elem->length-3, "%s", (char *) &elem->data);
+					if (elem->length <= 3) {
+						fprintf(stderr, "*** Warning - Corrupt CDP frame (possibly "
+								"an exploit attempt. (0x06 frame < 3)\n");
+						return;
+					}
+                    snprintf(ret_protoinfo->cdp.platform, elem->length-3, "%s", 
+							 (char *) &elem->data);
                 }
 
                 offset += elem->length;
@@ -907,7 +1164,7 @@ void GetProtoInfo(kis_packet *packet, packet_info *in_info) {
 
         // if it IS a turbocell packet, see if we can dissect it any...  Make sure its long
         // enough to have a SSID.
-        if (in_info->encrypted == 0 && packet->len > (unsigned int) (in_info->header_offset + LLC_OFFSET + 7)) {
+        if (in_info->encrypted == 0 && packet->len > (in_info->header_offset + LLC_OFFSET + 7)) {
             // Get the modes from the LLC header
             uint8_t turbomode = data[in_info->header_offset + LLC_OFFSET + 6];
             switch (turbomode) {
@@ -938,8 +1195,9 @@ void GetProtoInfo(kis_packet *packet, packet_info *in_info) {
                 in_info->turbocell_sat = 0;
 
             // Get the SSID
-            if (packet->len > (unsigned int) (in_info->header_offset + LLC_OFFSET + 26)) {
-                u_char *turbossid = &data[in_info->header_offset + LLC_OFFSET + 26];
+            if (packet->len > (in_info->header_offset + LLC_OFFSET + 26)) {
+                char *turbossid = (char *) &data[in_info->header_offset + 
+					LLC_OFFSET + 26];
                 if (isprint(turbossid[0])) {
                     // Make sure we get a terminator
                     int turbossidterm = 0;
@@ -952,9 +1210,10 @@ void GetProtoInfo(kis_packet *packet, packet_info *in_info) {
                         turbossidpos++;
                     }
 
-                    if (turbossidterm) {
-                        snprintf(in_info->ssid, SSID_SIZE, "%s", turbossid);
-                        MungeToPrintable(in_info->ssid, SSID_SIZE);
+					if (turbossidterm) {
+						snprintf(in_info->ssid, SSID_SIZE, "%s",
+								 MungeToPrintable(turbossid, 
+												  turbossidpos).c_str());
                     }
                 }
             }
@@ -1057,7 +1316,8 @@ void GetProtoInfo(kis_packet *packet, packet_info *in_info) {
 					case iapp_pdu_capability:
 						if (pdu_len != 1)
 							break;
-						in_info->wep = !!(pdu[3] & iapp_cap_wep);
+						if (!!(pdu[3] & iapp_cap_wep))
+							in_info->crypt_set |= crypt_wep;
 						break;
 					case iapp_pdu_announceint:
 						break;
@@ -1127,6 +1387,7 @@ void GetProtoInfo(kis_packet *packet, packet_info *in_info) {
 						uint8_t fchr = data[offset+x];
 						uint8_t schr = data[offset+x+1];
 
+						// We're already insulated against non A-Z characters
 						if (fchr < 'A' || fchr > 'Z' ||
 							schr < 'A' || schr > 'Z') {
 							ret_protoinfo->type = proto_udp;
@@ -1155,7 +1416,7 @@ void GetProtoInfo(kis_packet *packet, packet_info *in_info) {
 			unsigned int offset = in_info->header_offset + DHCPD_OFFSET + 252;
 
 
-			while (offset < packet->len) {
+			while ((offset + 1) < packet->len) {
 				if (data[offset] == 0x01) {
 					// netmask
 
@@ -1378,8 +1639,10 @@ void DecryptPacket(kis_packet *packet, packet_info *in_info,
 // We turn the packet into an "un-modified" packet for logging
 int MangleDeCryptPacket(const kis_packet *packet, const packet_info *in_info,
                         kis_packet *outpack, uint8_t *data, uint8_t *moddata) {
-    if (in_info->decoded == 0 || packet->error != 0 || packet->modified == 0)
-        return 0;
+
+    if (in_info->decoded == 0 || packet->error != 0 || 
+		packet->modified == 0 || in_info->header_offset < 4)
+		return 0;
 
     // Remove the WEP header
     outpack->ts.tv_sec = packet->ts.tv_sec;
@@ -1399,8 +1662,15 @@ int MangleDeCryptPacket(const kis_packet *packet, const packet_info *in_info,
     outpack->moddata = moddata;
     outpack->modified = 0;
 
-    // Copy the decrypted data, skipping the wep header and dropping the crc32 off the end
-    memcpy((void *) outpack->data, (void *) packet->moddata, in_info->header_offset - 4);
+	// Check the length
+	if (outpack->len < (in_info->header_offset - 4)) {
+		return 0;
+	}
+
+    // Copy the decrypted data, skipping the wep header and dropping the crc32 
+	// off the end
+    memcpy((void *) outpack->data, (void *) packet->moddata, 
+		   in_info->header_offset - 4);
     memcpy((void *) &outpack->data[in_info->header_offset - 4],
            (void *) &packet->moddata[in_info->header_offset],
            outpack->len - (in_info->header_offset - 4));
